#include "jpegdecoder.h"



JpegDecoder::JpegDecoder(ConcurrentQueue<Jpeg *, 256u> *queue, QObject *parent) :
    QObject(parent), m_queue(queue)
{
}

void JpegDecoder::operate()
{
    Jpeg *jpeg;
    forever {
        jpeg = m_queue->dequeue();

        struct jpeg_decompress_struct cinfo;
        struct error_mgr jerr;
        cinfo.err = jpeg_std_error(&jerr.mgr);
        jerr.mgr.error_exit = error_exit;
        if (setjmp(jerr.setjmp_buffer)) {
            jpeg_destroy_decompress(&cinfo);
        }
        jpeg_create_decompress(&cinfo);
        cinfo.do_fancy_upsampling = TRUE;
        cinfo.dct_method = JDCT_FLOAT;

        jpeg_mem_src(&cinfo, jpeg->data, jpeg->length);
        (void) jpeg_read_header(&cinfo, TRUE);
        (void) jpeg_start_decompress(&cinfo);

        JSAMPLE *samples = new JSAMPLE[cinfo.output_width * cinfo.output_height * cinfo.output_components];
        JSAMPROW *rows = new JSAMPROW[cinfo.output_height * cinfo.output_components];
        for(size_t i = 0; i < cinfo.output_height; i++) {
            rows[i] = samples + (i * cinfo.output_width * cinfo.output_components);
        }

        while (cinfo.output_scanline < cinfo.output_height) {
            jpeg_read_scanlines(&cinfo, rows + cinfo.output_scanline, 1);
        }
        (void) jpeg_finish_decompress(&cinfo);
        emit finished(samples, jpeg->x, jpeg->y, jpeg->width, jpeg->height);
        delete[] jpeg->data;
        delete jpeg;
        delete rows;
        jpeg_destroy_decompress(&cinfo);
    }
}
