#ifndef JPEGDECODER_H
#define JPEGDECODER_H

#include <QObject>
#include <QDebug>

// jpeglib requires stdlib.h
#include <stdio.h>
#include <jpeglib.h>
// setjmp.h is required for error handling
#include <setjmp.h>

#include "concurrentqueue.h"

struct Jpeg
{
    uint8_t *data;
    int length;
    int x;
    int y;
    int width;
    int height;
};

/*!
 * \brief Loads JPEG images
 *
 * Consumes
 */
class JpegDecoder : public QObject
{
    Q_OBJECT
public:
    JpegDecoder(ConcurrentQueue<Jpeg*, 256u> *queue, QObject *parent = 0);
    ConcurrentQueue<Jpeg*, 256u> *m_queue;
signals:
    void finished(const unsigned char *image, const quint32 x, const quint32 y, const quint32 width, const quint32 height);
public slots:
    void operate();
};

struct error_mgr
{
    struct jpeg_error_mgr mgr;
    jmp_buf setjmp_buffer;
};

typedef struct error_mgr* error_ptr;

METHODDEF(void)
error_exit (j_common_ptr cinfo)
{
  error_ptr myerr = (error_ptr) cinfo->err;
  (*cinfo->err->output_message) (cinfo);
  longjmp(myerr->setjmp_buffer, 1);
}

#endif // JPEGDECODER_H
