#include "vncrenderer.h"
#include <cstring>
#include "uploader.h"

VncRenderer::VncRenderer(QQuickWindow *window,
                         QOpenGLBuffer **vertices,
                         QOpenGLBuffer indices,
                         GLuint **texture,
                         unsigned int *drawCount,
                         QObject *parent) :
    QObject(parent), m_window(window), m_vertices(vertices), m_indices(indices), m_texture(texture), m_drawCount(drawCount), m_rendered(false)
{
    m_program.addShaderFromSourceFile(QOpenGLShader::Vertex, "draw_shader.vsh");
    m_program.addShaderFromSourceFile(QOpenGLShader::Fragment, "draw_shader.fsh");
    m_program.bind();
    m_program.setUniformValue("texture", 0);
    QMatrix4x4 ortho;
    ortho.ortho(QRect(QPoint(0, 0), QSize(2048, 2048)));
    m_program.setUniformValue("texortho", ortho);
    m_program.release();

    m_functions = QOpenGLContext::currentContext()->functions();
    m_functions->glActiveTexture(GL_TEXTURE0);
}

void VncRenderer::render()
{
    /*static QTime frameTime;
    qDebug() << qRound(1000.0 / frameTime.elapsed()) << *m_drawCount;
    frameTime.restart();*/

    if(*m_drawCount > 0) {
        m_program.bind();
        // m_functions->glActiveTexture(GL_TEXTURE0);
        (*m_vertices)->bind();
        m_indices.bind();
        m_functions->glVertexAttribPointer(m_program.attributeLocation("position"),
                                           2,
                                           GL_UNSIGNED_SHORT,
                                           GL_FALSE,
                                           sizeof(Vertex),
                                           (void*) 0);
        m_functions->glVertexAttribPointer(m_program.attributeLocation("color"),
                                           4,
                                           GL_UNSIGNED_BYTE,
                                           GL_TRUE,
                                           sizeof(Vertex),
                                           (void*) (2 * sizeof(GLushort)));
        m_functions->glVertexAttribPointer(m_program.attributeLocation("texcoord"),
                                           2,
                                           GL_UNSIGNED_SHORT,
                                           GL_FALSE,
                                           sizeof(Vertex),
                                           (void*) (2 * sizeof(GLushort) + 4 * sizeof(GLubyte)));
        m_program.enableAttributeArray("position");
        m_program.enableAttributeArray("color");
        m_program.enableAttributeArray("texcoord");
        m_functions->glBindTexture(GL_TEXTURE_2D, **m_texture);
        m_functions->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
        m_functions->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
        m_functions->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
        m_functions->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

        m_functions->glDrawElements(GL_TRIANGLE_STRIP, *m_drawCount, GL_UNSIGNED_SHORT, (void*) 0);
        *m_drawCount = 0;

        m_window->resetOpenGLState();
    }
}

QOpenGLFramebufferObject *VncRenderer::createFramebufferObject(const QSize &size)
{
    QMatrix4x4 ortho;
    ortho.ortho(QRect(QPoint(0, size.height()), QSize(size.width(), -1 * size.height())));
    m_program.bind();
    m_program.setUniformValue("ortho", ortho);
    m_program.release();

    QOpenGLFramebufferObjectFormat format;
    format.setTextureTarget(GL_TEXTURE_2D);
#ifdef GL_RGBA8
    format.setInternalTextureFormat(GL_RGBA8);
#else
    #ifdef RGBA8_OES
        format.setInternalTextureFormat(RGBA8_OES);
    #else
        format.setInternalTextureFormat(GL_RGBA);
    #endif
#endif

    QOpenGLFramebufferObject *framebufferObject = new QOpenGLFramebufferObject(size, format);
    emit FBOTextureChanged(framebufferObject->texture(), ortho);
    return framebufferObject;
}
